###############################################################################
# zapplet - A tray applet for interfacing with Zenoss
# Copyright (C) 2008 Nathaniel McCallum <nathaniel@natemccallum.com>
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
###############################################################################w

import sys
from threading import Thread

import gtk
import gobject

import zapplet.os

class Event(gobject.GObject):

	SEVERITY = [
		[ "clear",    "00FF00", "00CD00" ],
		[ "debug",    "C8C8C8", "AFAFAF" ],
		[ "info",     "B3B3FF", "5566FF" ],
		[ "warning",  "FFF280", "FFDA00" ],
		[ "error",    "FFBA75", "FF8822" ],
		[ "critical", "F88880", "EE0000" ],
	]

	def _asyncmethod(func):
		def x(*args):
			if hasattr(args[0], "_async"):
				async = args[0]._async
			else:
				async = list(args).pop()

			def doit(callback, argv):
				"Do the action and schedule the results into the main loop."
				def on_results(r, e=None):
					"Callback in the mainloop, just wraps the callback in locks."
					gtk.gdk.threads_enter()
					async(r, e)
					gtk.gdk.threads_leave()
					return False

				try:
					results = callback(*argv)
					gobject.idle_add(on_results, results, None)
				except:
					gobject.idle_add(on_results, None, sys.exc_info())

			if async:
				Thread(target=doit, args=(func, args)).start()
			else:
				return func(*args)
		return x

	@classmethod
	def _get_base_url(cls, server, auth=False):
		osf = zapplet.os.OSFunctions.factory()
		return 'http%s://%s@%s:%d' % (
			osf.get(server, "ssl") and "s" or "", 
			auth and osf.get(server, "username") + ":" + osf.get(server, "password") or "",
			osf.get(server, "hostname"),
			osf.get(server, "port")
		)

	@classmethod
	def _get_xmlrpc_proxy(cls, server, obj):
		import xmlrpclib
		return xmlrpclib.ServerProxy(cls._get_base_url(server, True) + obj)
	
	@classmethod
	def _openURL(cls, server, path):
		import webbrowser
		webbrowser.open(cls._get_base_url(server) + path)

	def __eq__(self, other):
		return self.evid == other.evid

	def __init__(self, server, keys, async=None):
		for k, v in keys.items():
			setattr(self, k, v)
		self.server = server
		self._async = async

	@classmethod
	def view_event_console(cls, server):
		return cls._openURL(server, "/zport/dmd/Events/viewEvents?notabs=1")
	
	@classmethod
	@_asyncmethod
	def load_events_from_server(cls, server, async=None):
		xmlrpc = cls._get_xmlrpc_proxy(server, "/zport/dmd/ZenEventManager")

		# Get event details
		events = xmlrpc.getEventList([], "", "severity DESC, eventState ASC, lastTime DESC")

		# Create summary pseudo-event
		results  = sorted(xmlrpc.getEventSummary())
		highest  = filter(lambda x: x[-1] > 0, results)[-1]
		severity = int(highest[0].split("_")[1])
		data = {
			"count":    highest[-1],
			"eventState": 0,
			"severity": severity,
			"summary": "%d %s events awaiting resolution on server '%s'!" % \
				(highest[-1], cls.SEVERITY[severity][0], zapplet.os.OSFunctions.factory().get(server, "hostname")),
		}
		return (Event(server, data), [Event(server, e, async) for e in events])

	@_asyncmethod
	def add_log(self, message, async=None):
		xmlrpc = self.__class__._get_xmlrpc_proxy(self.server, "/zport/dmd/ZenEventManager")
		return xmlrpc.manage_addLogMessage(self.evid, message)

	@_asyncmethod
	def acknowledge(self, async=None):
		import xmlrpclib
		try:
			xmlrpc = self.__class__._get_xmlrpc_proxy(self.server, "/zport/dmd/ZenEventManager")
			result = xmlrpc.manage_ackEvents((self.evid,))
			self.eventState = 1
			return result
		except xmlrpclib.ProtocolError, e: # Ignore the 302 that is thrown, this call works anyway
			if e.errcode != 302:
				raise

	@_asyncmethod
	def historify(self, async=None):
		xmlrpc = self.__class__._get_xmlrpc_proxy(self.server, "/zport/dmd/ZenEventManager")
		return xmlrpc.manage_deleteEvents((self.evid,))

	def view(self):
		return self.__class__._openURL(self.server, "/zport/dmd/Events/viewEventFields?evid=" + self.evid)

	def get_pixbuf(self, width, height):
		import cairo
		import math

		size = min(width, height)
		line = size / 64.0
		ack  = self.eventState > 0

		# Create our drawing area
		cis = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
		c   = cairo.Context(cis)
		c.set_line_width(line)
		c.select_font_face("sans-serif", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_NORMAL)

		# TODO: Drop shadow?

		# Create our gradient for the colored circle
		pat = cairo.LinearGradient(width/2, line, width/2, height - line)

		# Start (top)
		color = self.__class__.SEVERITY[self.severity][1]
		r = int(color[0:2], 16) / 255.0
		g = int(color[2:4], 16) / 255.0
		b = int(color[4:6], 16) / 255.0
		pat.add_color_stop_rgba(0, r, g, b, ack and 0.5 or 1)

		# Stop (bottom)
		color = self.__class__.SEVERITY[self.severity][2]
		r = int(color[0:2], 16) / 255.0
		g = int(color[2:4], 16) / 255.0
		b = int(color[4:6], 16) / 255.0
		pat.add_color_stop_rgba(1, r, g, b, ack and 0.5 or 1)

		# Draw the colored circle
		c.set_source(pat)
		c.arc(size / 2, size / 2, size / 2 - line, 0, 2 * math.pi)
		c.fill()

		# Draw the black outline
		c.set_source_rgba(0, 0, 0, ack and 0.5 or 1)
		c.arc(size / 2, size / 2, size / 2 - line, 0, 2 * math.pi)
		c.stroke()

		# Determine what font size to use
		for i in range(size):
			w, h = c.text_extents("HHH")[2:4]
			
			# If the text no longer fits in the shape, break
			if math.sqrt(pow(w/2, 2) + pow(h/2, 2)) >= (size / 2 - line) * .7:
				break
			c.set_font_size(i)

		# Draw our text centered in the circle
		c.set_source_rgba(1, 1, 1, 1)
		xb, yb, w, h = c.text_extents(str(min(self.count, 999)))[0:4]
		c.move_to(width / 2 - w / 2 - xb - 0.25, height / 2 - h / 2 - yb - 0.25)
		c.show_text(str(min(self.count, 999)))
		c.stroke()

		# Change the byte order for GDK
		data    = cis.get_data()
		newdata = ""
		for i in range(len(data) / 4):
			newdata += ord(data[i*4+3]) and chr(int(ord(data[i*4+2])/(ord(data[i*4+3])/255.0))) or chr(0)
			newdata += ord(data[i*4+3]) and chr(int(ord(data[i*4+1])/(ord(data[i*4+3])/255.0))) or chr(0)
			newdata += ord(data[i*4+3]) and chr(int(ord(data[i*4+0])/(ord(data[i*4+3])/255.0))) or chr(0)
			newdata += data[i*4+3]

		# Return a pixbuf
		return gtk.gdk.pixbuf_new_from_data(newdata, gtk.gdk.COLORSPACE_RGB, True, 8, width, height, cis.get_stride())
